@@ -6,36 +6,25 @@ use warnings;
 use base qw/Exporter/;
 use Carp qw/croak carp/;
 
-our $VERSION = '0.07';
+
+our $VERSION = '0.09';
 our @EXPORT  = qw/convert convertor addequal UL_CHR UL_ENT UL_EQV UL_SEQ UL_7BT UL_ALL/;
-our %EXPORT_TAGS = ( utils => [grep{ !/^UL_/ }@EXPORT], flags => [grep{/^UL_/}@EXPORT] );
+our %EXPORT_TAGS = (
+	utils => [grep{!/^UL_/}@EXPORT],
+	flags => [grep{ /^UL_/}@EXPORT]
+);
 
-#use enum qw/BITMASK: RP_CHR RP_ENT EQ_CHR EQ_SEQ EQ_7BT/;
-use constant RP_CHR => 0x01;
-use constant RP_ENT => 0x02;
-use constant EQ_CHR => 0x04;
-use constant EQ_SEQ => 0x08;
-use constant EQ_7BT => 0x10;
 
-use constant UL_CHR => RP_CHR;          # REPLACE TO CHAR   (default <Space>)
-use constant UL_ENT => RP_CHR | RP_ENT; # REPLACE TO ENTITY (like    &#0000;)
+use enum qw/BITMASK: RP_CHR RP_ENT EQ_CHR EQ_SEQ EQ_7BT/;
+use enum qw/nil src dst all/;
+use constant uni => qr/^(?:utf16|utf8|utf7|ucs4|uchr|uhex|latin1)$/;
+use constant UL_CHR => RP_CHR;          # REPLACE TO CHAR   (default <?>)
+use constant UL_ENT => RP_CHR | RP_ENT; # REPLACE TO ENTITY (like &#0000;)
 use constant UL_EQV => EQ_CHR;          # EQUIVALENT char
 use constant UL_SEQ => EQ_CHR | EQ_SEQ; # EQUIVALENT sequence of chars
 use constant UL_7BT => EQ_7BT | UL_SEQ; # EQUIVALENT sequence of 7bit chars
 use constant UL_ALL => UL_CHR | UL_ENT | UL_EQV | UL_SEQ;
-
-#use enum qw/nil dst src all/;
-use constant nil => 0x0;
-use constant src => 0x1;
-use constant dst => 0x2;
-use constant all => 0x3;
-use constant uni => qr/^utf16|utf8|utf7|ucs4|uchr|uhex|latin1$/;
-
-our %MAPPING;
-our %CONVERT;
-our %EQUIVAL;
-our $REGISTR;
-our $TEST= 0;
+our (%MAPPING, %CONVERT, %EQUIVAL, $REGISTR, $TEST);
 
 sub convertor($$;$$)
 {
@@ -47,13 +36,11 @@ sub convertor($$;$$)
 
 	my  ($SRC, $DST) = ($src, $dst);
 	for ($SRC, $DST){
-		next if $_=~uni or s/^ucs2|unicode$/utf16/o or s/^iso-8859-1$/latin1/o;
+		next if $_=~uni or s/^(?:ucs2|unicode)$/utf16/o or s/^iso-8859-1$/latin1/o;
 		next if exists $MAPPING{$_};
-		unless ($REGISTR){ require Unicode::Map; local $_; 
-				$REGISTR = new Unicode::Map();
-		}
-		$_ = lc $REGISTR->id( uc $_ ) ||
-			croak "Character Set '$_' not defined!";
+		unless ($REGISTR){ require Unicode::Map; local $_;
+				$REGISTR = new Unicode::Map() }
+		$_ = lc $REGISTR->id(uc $_) || croak "Character Set '$_' not defined!";
 		$_ = 'latin1' if $_ eq 'iso-8859-1';
 	}
 
@@ -66,7 +53,7 @@ sub convertor($$;$$)
 	for ([$src, $SRC, $map&src], [$dst, $DST, $map&dst]){
 		next unless $$_[2] and !$MAPPING{$$_[0]};
 		$MAPPING{$$_[0]} = $MAPPING{$$_[1]} ||
-	   ($MAPPING{$$_[1]} = new Unicode::Map($$_[1])) ||
+	   ($MAPPING{$$_[1]} = new Unicode::Map(uc $$_[1])) ||
 		croak "Can't create Unicode::Map object for '$$_[1]' charset!";
 	}
 
@@ -99,7 +86,7 @@ sub convertor($$;$$)
 		$mut = '$_='.$mut;
 	}
 	else{ $mut = __sbcs_convertor($SRC, $DST, $mod, $chr) }
-	warn "$SRC -> $DST ($map)\t\t$mut\n" if $TEST;
+	warn "MUTATOR: $SRC -> $DST [$mod]\t$mut\n" if $TEST;
 
 	return
 		$CONVERT{$src}{$dst}{$mod}{$chr} =
@@ -112,7 +99,7 @@ sub convertor($$;$$)
 
 sub convert($$;$$$){
 	my $fn = convertor( shift, shift, $_[1], $_[2] );
-	goto &$fn;
+	&$fn;
 }
 
 sub addequal(@)
@@ -143,29 +130,17 @@ sub __sbcs_convertor($$$$)
 		$mod & RP_CHR and not $mod & RP_ENT;
 
 	# fill charsets arrays with U+0000
-	for ([$src, \@src], ($mod&EQ_7BT)?():[$dst, \@dst]){
+	@dst = (0) x 0x80 if $mod & EQ_7BT;
+	for ([$src, \@src], ($mod & EQ_7BT)?():[$dst, \@dst]){
 		my $conv = convertor( $$_[0], 'utf16' );
 		@{$$_[1]} = map {&$conv(); $_ ? unpack 'n', $_ : 0} map chr, 0x80..0xff;
 	}
 
-	my $find = sub(){
-		my $chr = $src[$_];
-		return 0 unless exists $EQUIVAL{$chr};
-		LOOP:
-		for (@{$EQUIVAL{$chr}}){
-			if (!ref){ next LOOP unless $_ < 0x80 or exists $dst{$_}; return $_ }
-			next unless $mod & EQ_SEQ;
-			for (@$_){ next LOOP unless $_ < 0x80 or exists $dst{$_}} return $_;
-		}
-		return 0;
-	};
-
-	@dst = (0) x 0x80        if $mod & EQ_7BT;
-	@src{@src} = 0x80..0xff  if $mod &~RP_CHR;
+	@src{@src} = 0x80..0xff  if $mod & ~RP_CHR;
 	@dst{@dst} = 0x80..0xff;
 
 	# collect positions of unused chars
-	if ($mod & ~UL_CHR){                # if need indirect replace
+	if ($mod & ~RP_CHR){                # if need indirect replace
 		for (0 .. $#dst){
 		push @dif, $_ + 0x80 if
 			!$dst[$_] or                # char not used in dst codepage
@@ -179,6 +154,18 @@ sub __sbcs_convertor($$$$)
 		while (<DATA>){ s/\s*#.*//so; addequal($_); }
 	}
 
+	my $find = sub(){
+		my $chr = $src[$_];
+		return undef unless exists $EQUIVAL{$chr};
+		LOOP:
+		for (@{$EQUIVAL{$chr}}){
+			if (!ref){ next LOOP unless $_ < 0x80 or exists $dst{$_}; return $_ }
+			next unless $mod & EQ_SEQ;
+			for (@$_){ next LOOP unless $_ < 0x80 or exists $dst{$_}} return $_;
+		}
+		return undef;
+	};
+
 	my (@map, @eqv, @ent, @chr, @del);
 
 	for (0 .. $#src)
@@ -190,9 +177,11 @@ sub __sbcs_convertor($$$$)
 			push @map, [$_, $src[$_]];
 
 		}elsif( $mod & EQ_CHR and my $uni = &$find ){
-			next if not ref $uni and
-			push @map, [$_, $uni];
+			next if     ref $uni and
 			push @eqv, [$_, $uni];
+			next if not ($dst{$uni} and $_ == $dst{$uni} - 0x80) and
+			push @map, [$_, $uni];
+            @dif = grep{ $_ != $dst{$uni} }@dif;
 
 		}elsif( $mod & RP_ENT ){
 			push @ent, [$_, $src[$_]];
@@ -365,7 +354,7 @@ http://www.perl.com/CPAN
 
 =head1 SEE ALSO
 
-Unicode::String, Unicode::Map.
+Unicode::String, Unicode::Map, map
 
 =cut
 
@@ -484,6 +473,11 @@ __DATA__
 215D 35+2F+38 # 5/8
 215E 37+2F+38 # 7/8
 
+2013 0096 2D  # EN DASH   -
+2014 0097 2D  # EM DASH   -
+0096 2013 2D  # EN DASH   -
+0097 2014 2D  # EM DASH   -
+
 00BF 3F   # INVERTED      ?
 00A8 22   # DIAERESIS     "
 00D7 78   # MULTIPLY      x
@@ -494,8 +488,6 @@ __DATA__
 2219 2E   # BULLET       .
 00B7 2E   # MIDDLE DOT   .
 02dc 7E   # SMALL TILDE   ~
-2013 2D   # EN DASH       -
-2014 2D   # EM DASH       -
 2018 27   # SINGLE LEFT '
 2019 27   # SINGLE RIGH '
 201A 27   # SINGLE LOW9 '
@@ -510,6 +502,7 @@ __DATA__
 2264 3C+3D# LESS | EQUAL <=
 2265 3E+3D# GREAT| EQUAL >=
 203C 21+21# 2 EXCLAMATION!!
+203D 3F+21#              ?!
 
 # BLOCK
 2588 42   #  B
